@@ -20,6 +20,7 @@ from anyscale.util import (
     get_cluster_config,
     get_requirements,
     send_json_request,
+    Timer,
 )
 
 
@@ -92,11 +93,37 @@ class SnapshotRunner:
                 snapshot_uuid,
                 "--tag",
                 "project_dir=" + project_dir,
+                "--verbose",
+                "--verbose",
             ],
             env=new_env,
-            stderr=STDOUT,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
         )
-        proc.communicate()
+        collecting_output = False
+        while True:
+            if proc.stdout:
+                line = proc.stdout.readline()
+                if not line:
+                    break
+
+                if b"open repository" in line:
+                    timer = Timer("Taking snapshot of all files within this project")
+                    timer.start()
+                elif b"start backup" in line:
+                    timer.stop()
+                    timer = Timer("Saving snapshot to Anyscale")
+                    timer.start()
+                elif b"scan finished" in line:
+                    timer.stop()
+                    print("\n" + line.decode())
+                elif not collecting_output and line == b"\n":
+                    collected_out = ""
+                    collecting_output = True
+
+                if collecting_output:
+                    collected_out += line.decode()
+        print(collected_out)
 
     def download(
         self, name: str, target_directory: Union[str, Path], creds: Dict[str, str],
@@ -370,22 +397,36 @@ def create_snapshot(
         )
     else:
         requirements = ""
-
-    resp = send_json_request(
-        "snapshot_create",
-        {
-            "project_id": project_id,
-            "project_config": json.dumps(project_definition.config),
-            "cluster_config": json.dumps(cluster_config),
-            "requirements": requirements,
-            "description": description if description else "",
-            "tags": tags,
-        },
-        method="POST",
-    )
-
-    snapshot_uuid: str = resp["uuid"]
-    creds = resp["credentials"]
+    if TEST_V2:
+        resp = send_json_request(
+            "/api/v2/snapshots/",
+            {
+                "project_id": project_id,
+                "project_config": json.dumps(project_definition.config),
+                "cluster_config": json.dumps(cluster_config),
+                "requirements": requirements,
+                "description": description if description else "",
+                "tags": tags,
+            },
+            method="POST",
+        )
+        snapshot_uuid: str = resp["result"]["id"]
+        creds = resp["result"]["credentials"]
+    else:
+        resp = send_json_request(
+            "snapshot_create",
+            {
+                "project_id": project_id,
+                "project_config": json.dumps(project_definition.config),
+                "cluster_config": json.dumps(cluster_config),
+                "requirements": requirements,
+                "description": description if description else "",
+                "tags": tags,
+            },
+            method="POST",
+        )
+        snapshot_uuid = resp["uuid"]
+        creds = resp["credentials"]
 
     try:
         snapshot_runner.upload(
