@@ -3,7 +3,10 @@ import json
 import logging
 import os
 import re
-from typing import Any, Dict, List
+import sys
+import threading
+import time
+from typing import Any, Dict, Iterator, List
 import unicodedata
 
 import boto3
@@ -50,8 +53,14 @@ def load_credentials() -> str:
     with open(path) as f:
         try:
             credentials: Dict[str, str] = json.load(f)
-        except json.JSONDecodeError as e:
-            raise click.ClickException(e)  # type: ignore
+        except json.JSONDecodeError:
+            msg = (
+                "Unable to load user credentials.\n\nTip: Try creating your "
+                "user credentials again by going to {}/credentials and "
+                "following the instructions. If this does not work, "
+                "please contact Anyscale support!".format(anyscale.conf.ANYSCALE_HOST)
+            )
+            raise click.ClickException(msg)
     if "cli_token" not in credentials:
         raise click.ClickException("Credential file not valid, please regenerate it.")
     return credentials["cli_token"]
@@ -72,6 +81,8 @@ def send_json_request(
             resp = requests.post(url, json=json_args, cookies=cookies)
         elif method == "DELETE":
             resp = requests.delete(url, json=json_args, cookies=cookies)
+        elif method == "PATCH":
+            resp = requests.patch(url, data=json_args, cookies=cookies)
         else:
             assert "unknown method {}".format(method)
     except requests.exceptions.ConnectionError:
@@ -92,20 +103,23 @@ def send_json_request(
     return json_resp
 
 
+# DEPRECATED
 def serialize_datetime(d: datetime.datetime) -> str:
     # Make sure that overwriting the tzinfo in the line below is fine.
     # Note that we have to properly convert the timezone if one is
     # already specified. This can be done with the .astimezone method.
-    assert d.tzinfo is None
-    return d.replace(tzinfo=datetime.timezone.utc).isoformat()
+    if d.tzinfo is None:
+        return d.replace(tzinfo=datetime.timezone.utc).isoformat()
+    else:
+        return d.astimezone(datetime.timezone.utc).isoformat()
 
 
 def deserialize_datetime(s: str) -> datetime.datetime:
-    return datetime.datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%f+00:00")
+    return datetime.datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%f%z")
 
 
 def humanize_timestamp(timestamp: datetime.datetime) -> str:
-    delta = datetime.datetime.utcnow() - timestamp
+    delta = datetime.datetime.now(datetime.timezone.utc) - timestamp
     offset = float(delta.seconds + (delta.days * 60 * 60 * 24))
     delta_s = int(offset % 60)
     offset /= 60
@@ -205,3 +219,41 @@ def get_available_regions() -> List[str]:
     boto_session = boto3.session.Session()
     client = boto_session.client("ec2")
     return [region["RegionName"] for region in client.describe_regions()["Regions"]]
+
+
+class Timer:
+    """
+    Code adopted from https://stackoverflow.com/a/39504463/3727678
+
+    Spawn thread and time process that may be blocking.
+    """
+
+    def timer_generator(self) -> Iterator[str]:
+        while True:
+            time_diff = time.gmtime(time.time() - self.start_time)
+            yield "{0}: {1}".format(self.message, time.strftime("%M:%S", time_diff))
+
+    def __init__(self, message: str = "") -> None:
+        self.message = message
+        self.busy = False
+        self.start_time = 0.0
+
+    def timer_task(self) -> None:
+        while self.busy:
+            sys.stdout.write(next(self.timer_generator()))
+            sys.stdout.flush()
+            time.sleep(0.1)
+            sys.stdout.write("\b" * (len(self.message) + 20))
+            sys.stdout.flush()
+
+    def start(self) -> None:
+        self.busy = True
+        self.start_time = time.time()
+        threading.Thread(target=self.timer_task).start()
+
+    def stop(self) -> None:
+        sys.stdout.write("\n")
+        sys.stdout.flush()
+        self.busy = False
+        self.start_time = 0.0
+        time.sleep(1)
